/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "ButtonTexture.h"

ButtonTexture::ButtonTexture()
{
	useAlphaMap = false;

	memoryEntryDC = 0;
	memoryEntryBitmap = 0;

	memorySelEntryDC = 0;
	memorySelEntryBitmap = 0;

	memoryFolderDC = 0;
	memoryFolderBitmap = 0;

	memorySelFolderDC = 0;
	memorySelFolderBitmap = 0; 

	memoryInfoDC = 0;
	memoryInfoBitmap = 0; 

	memorySeparatorDC = 0;
	memorySeparatorBitmap = 0; 

	memoryArrowDC = 0;
	memoryArrowBitmap = 0; 

	memorySelArrowDC = 0;
	memorySelArrowBitmap = 0; 

	imageEntryHeight = 20;
	imageFolderHeight = imageEntryHeight;
	imageInfoHeight = imageEntryHeight;
	imageSeparatorHeight = 2;

	defaultentry.trueTransparent = false;

	defaultentry.button_saturation = 255;
	defaultentry.button_hueIntensity = 0;
	defaultentry.button_hueColor = RGB(255, 255, 255);

	defaultentry.borderMethod = 0;

	defaultentry.leftEdge = 0;
	defaultentry.topEdge = 0;
	defaultentry.rightEdge = 0;
	defaultentry.bottomEdge = 0;

	defaultentry.mode = MULTIBLT_STRETCH;

	arrowSpacing = 4;
	arrowOnLeft = false;
}

ButtonTexture::~ButtonTexture()
{
	EntryGradient.clear();
	SelEntryGradient.clear();
	FolderGradient.clear();
	SelFolderGradient.clear();
	InfoGradient.clear();
	SeparatorGradient.clear();

	if(memoryEntryBitmap)
	{
		memoryEntryBitmap = (HBITMAP) SelectObject(memoryEntryDC, memoryEntryBitmap);
		DeleteObject(memoryEntryBitmap);
	}
	if(memoryEntryDC)
		DeleteDC(memoryEntryDC);

	if(memorySelEntryBitmap)
	{
		memorySelEntryBitmap = (HBITMAP) SelectObject(memorySelEntryDC, memorySelEntryBitmap);
		DeleteObject(memorySelEntryBitmap);
	}
	if(memorySelEntryDC)
		DeleteDC(memorySelEntryDC);

	if(memoryFolderBitmap)
	{
		memoryFolderBitmap = (HBITMAP) SelectObject(memoryFolderDC, memoryFolderBitmap);
		DeleteObject(memoryFolderBitmap);
	}
	if(memoryFolderDC)
		DeleteDC(memoryFolderDC);

	if(memorySelFolderBitmap)
	{
		memorySelFolderBitmap = (HBITMAP) SelectObject(memorySelFolderDC, memorySelFolderBitmap);
		DeleteObject(memorySelFolderBitmap);
	}
	if(memorySelFolderDC)
		DeleteDC(memorySelFolderDC);

	if(memoryInfoBitmap)
	{
		memoryInfoBitmap = (HBITMAP) SelectObject(memoryInfoDC, memoryInfoBitmap);
		DeleteObject(memoryInfoBitmap);
	}
	if(memoryInfoDC)
		DeleteDC(memoryInfoDC);

	if(memorySeparatorBitmap)
	{
		memorySeparatorBitmap = (HBITMAP) SelectObject(memorySeparatorDC, memorySeparatorBitmap);
		DeleteObject(memorySeparatorBitmap);
	}
	if(memorySeparatorDC)
		DeleteDC(memorySeparatorDC);

	if(memoryArrowBitmap)
	{
		memoryArrowBitmap = (HBITMAP) SelectObject(memoryArrowDC, memoryArrowBitmap);
		DeleteObject(memoryArrowBitmap);
	}
	if(memoryArrowDC)
		DeleteDC(memoryArrowDC);

	if(memorySelArrowBitmap)
	{
		memorySelArrowBitmap = (HBITMAP) SelectObject(memorySelArrowDC, memorySelArrowBitmap);
		DeleteObject(memorySelArrowBitmap);
	}
	if(memorySelArrowDC)
		DeleteDC(memorySelArrowDC);
}

void ButtonTexture::parseCFG(string cfg, COLORREF &bgcolor, COLORREF &lightcolor, COLORREF &darkcolor)
{
	bgcolor = RGB(255,255,255);
	lightcolor = GetSysColor(COLOR_3DLIGHT);
	darkcolor = GetSysColor(COLOR_3DSHADOW);
		
	if ( cfg.empty() )
		return;

	int color;
	char main[32], light[32], dark[32];
	char *buffers[] = {main, light, dark};

	int numTokens = LCTokenize(cfg.c_str(), buffers, 3, NULL);
		
	if(numTokens == 1)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 2)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
	else if (numTokens == 3)
	{
		color = strtol(main, NULL, 16);
		bgcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(light, NULL, 16);
		lightcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));

		color = strtol(dark, NULL, 16);
		darkcolor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
	}
}

void ButtonTexture::parseGradientCFG(string cfg, int imageType)
{
	if ( cfg.empty() )
		return;

	int color;
	COLORREF insertColor;
	char token[4096];
	LPCSTR nextToken = cfg.c_str();
	
	while (GetToken(nextToken, token, &nextToken, false))
	{
		color = strtol(token, NULL, 16);
		insertColor = RGB(GetBValue(color), GetGValue(color), GetRValue(color));
		if (imageType == 1)
		{
			EntryGradient.insert(EntryGradient.end(), insertColor );
		}
		else if (imageType == 2)
		{
			SelEntryGradient.insert(SelEntryGradient.end(), insertColor );
		}
		else if (imageType == 3)
		{
			FolderGradient.insert(FolderGradient.end(), insertColor );
		}
		else if (imageType == 4)
		{
			SelFolderGradient.insert(SelFolderGradient.end(), insertColor );
		}
		else if (imageType == 5)
		{
			InfoGradient.insert(InfoGradient.end(), insertColor );
		}
		else if (imageType == 10)
		{
			SeparatorGradient.insert(SeparatorGradient.end(), insertColor );
		}
	}
}

void ButtonTexture::subconfigure(string type, BUTTONSETTINGS &workSettings, BUTTONSETTINGS &defaultSettings)
{
	NameValuePair modeConstants[] =
	{
		{ "stretch", MULTIBLT_STRETCH },
		{ "tile", MULTIBLT_TILE },
		{ "tile-horizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tile-vertical", MULTIBLT_TILEVERTICAL },
		{ "tilehorizontal", MULTIBLT_TILEHORIZONTAL },
		{ "tilevertical", MULTIBLT_TILEVERTICAL },
		{ 0, 0 }
	};

	NameValuePair borderValues[] = 
	{
		{ ".none", 0 },
		{ "none", 0 },
		{ "raised", 1 },
		{ "sunken", 2 },
		{ "etched", 3 },
		{ "bump", 4 },
		{ 0, 0 }
	};

	workSettings.button_saturation = GetxRCInt(type , "SaturationIntensity", defaultSettings.button_saturation, 0, 255);
	workSettings.button_hueIntensity = GetxRCInt(type , "HueIntensity", defaultSettings.button_hueIntensity, 0, 255);
	workSettings.button_hueColor = GetxRCColor(type, "HueColor", defaultSettings.button_hueColor);

	workSettings.borderMethod = GetxRCNamedValue(type, "BorderMethod", borderValues, defaultSettings.borderMethod);

	workSettings.leftEdge = GetxRCInt(type, "ImageLeftEdge", defaultSettings.leftEdge, 0);
	workSettings.topEdge = GetxRCInt(type, "ImageTopEdge", defaultSettings.topEdge, 0);
	workSettings.rightEdge = GetxRCInt(type, "ImageRightEdge", defaultSettings.rightEdge, 0);
	workSettings.bottomEdge = GetxRCInt(type, "ImageBottomEdge", defaultSettings.bottomEdge, 0);

	workSettings.mode = GetxRCNamedValue(type, "ImageMode", modeConstants, defaultSettings.mode);

	workSettings.trueTransparent = GetxRCBool(type, "TrueTransparent", defaultSettings.trueTransparent);
}

void ButtonTexture::configure(const string &name)
{
	if ( IsOS(OS_2KXP) && !GetxRCBool(name, "UseSolidColors") && !GetxRCString(name, "Image", "").empty() )
		useAlphaMap = GetxRCBool(name, "AlphaMap");

	activeFolderUseFolderDefault = GetxRCBool(name, "ActiveFolderUseFolderDefault");


	subconfigure((name + "Entry"), entry, defaultentry);

	subconfigure((name + "ActiveEntry"), selentry, entry);

	subconfigure((name + "Folder"), folder, entry);

	subconfigure((name + "ActiveFolder"), selfolder, (activeFolderUseFolderDefault ? folder : selentry));

	subconfigure((name + "Info"), info, defaultentry);

	subconfigure((name + "Separator"), separator, defaultentry);


	useentrySolidColors = GetxRCBool(name, "UseEntrySolidColors");

	if (useentrySolidColors)
	{
		entrysolidcolors = GetxRCLine(name, "EntrySolidColors", "");
		if ( !entrysolidcolors.empty() )
			parseCFG(entrysolidcolors, entrybackgroundColor, entrylightBevelColor, entrydarkBevelColor);
		else
		{
			entrybackgroundColor = GetSysColor(COLOR_BTNFACE);
			entrylightBevelColor = GetSysColor(COLOR_3DLIGHT);
			entrydarkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		EntryGradient.insert(EntryGradient.end(), entrybackgroundColor );

		entrybevelSize = GetxRCInt(name, "EntrySolidBevelSize", 0, 0);

		entrygradientColor = GetxRCLine(name, "EntrySolidGradientColors", "");
	
		parseGradientCFG(entrygradientColor, 1);
		
		entryverticalGradient = GetxRCBool(name, "EntrySolidGradientVertical");
	}
	else
	{
		entryImage = GetxRCString(name, "EntryImage", "");
		if (!entryImage.empty())
		{
			memoryEntryBitmap = LoadLSImage(entryImage.c_str(), 0);

			if (memoryEntryBitmap && useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryEntryBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryEntryBitmap, entry.button_saturation, entry.button_hueIntensity, entry.button_hueColor);
			}

			GetLSBitmapSize(memoryEntryBitmap, &imageEntryWidth, &imageEntryHeight);

			memoryEntryDC = CreateCompatibleDC(NULL);
			SelectObject(memoryEntryDC, memoryEntryBitmap);
		}
	}

	useselentrySolidColors = GetxRCBool(name, "UseActiveEntrySolidColors", useentrySolidColors);

	if (useselentrySolidColors)
	{
		selentrysolidcolors = GetxRCLine(name, "ActiveEntrySolidColors", entrysolidcolors);
		if ( !selentrysolidcolors.empty() )
			parseCFG(selentrysolidcolors, selentrybackgroundColor, selentrylightBevelColor, selentrydarkBevelColor);
		else
		{
			selentrybackgroundColor = GetSysColor(COLOR_BTNHIGHLIGHT);
			selentrylightBevelColor = GetSysColor(COLOR_3DHILIGHT);
			selentrydarkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		SelEntryGradient.insert(SelEntryGradient.end(), selentrybackgroundColor );

		selentrybevelSize = GetxRCInt(name, "ActiveEntrySolidBevelSize", entrybevelSize, 0);

		parseGradientCFG(GetxRCLine(name, "ActiveEntrySolidGradientColors", entrygradientColor), 2);
		
		selentryverticalGradient = GetxRCBool(name, "ActiveEntrySolidGradientVertical", entryverticalGradient);
	}
	else
	{
		selentryImage = GetxRCString(name, "ActiveEntryImage", entryImage);
		if (!selentryImage.empty())
		{
			memorySelEntryBitmap = LoadLSImage(selentryImage.c_str(), 0);

			if (memorySelEntryBitmap && useAlphaMap)
			{
				BITMAP bm;
				GetObject(memorySelEntryBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memorySelEntryBitmap, selentry.button_saturation, selentry.button_hueIntensity, selentry.button_hueColor);
			}

			GetLSBitmapSize(memorySelEntryBitmap, &imageSelEntryWidth, &imageSelEntryHeight);

			memorySelEntryDC = CreateCompatibleDC(NULL);
			SelectObject(memorySelEntryDC, memorySelEntryBitmap);
		}
	}

	usefolderSolidColors = GetxRCBool(name, "UseFolderSolidColors", useentrySolidColors);

	if (usefolderSolidColors)
	{
		foldersolidcolors = GetxRCLine(name, "FolderSolidColors", entrysolidcolors);
		if ( !foldersolidcolors.empty() )
			parseCFG(foldersolidcolors, folderbackgroundColor, folderlightBevelColor, folderdarkBevelColor);
		else
		{
			folderbackgroundColor = GetSysColor(COLOR_BTNFACE);
			folderlightBevelColor = GetSysColor(COLOR_3DLIGHT);
			folderdarkBevelColor = GetSysColor(COLOR_3DSHADOW);
		}
		//BG Insert in Gradient List
		FolderGradient.insert(FolderGradient.end(), folderbackgroundColor );

		folderbevelSize = GetxRCInt(name, "FolderSolidBevelSize", entrybevelSize, 0);

		parseGradientCFG(GetxRCLine(name, "FolderSolidGradientColors", entrygradientColor), 3);

		folderverticalGradient = GetxRCBool(name, "FolderSolidGradientVertical", entryverticalGradient);
	}
	else
	{
		folderImage = GetxRCString(name, "FolderImage", entryImage);
		if (!folderImage.empty())
		{
			memoryFolderBitmap = LoadLSImage(folderImage.c_str(), 0);

			if (memoryFolderBitmap && useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryFolderBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryFolderBitmap, folder.button_saturation, folder.button_hueIntensity, folder.button_hueColor);
			}

			GetLSBitmapSize(memoryFolderBitmap, &imageFolderWidth, &imageFolderHeight);

			memoryFolderDC = CreateCompatibleDC(NULL);
			SelectObject(memoryFolderDC, memoryFolderBitmap);
		}
	}

	useselfolderSolidColors = GetxRCBool(name, "UseActiveFolderSolidColors", (activeFolderUseFolderDefault ? usefolderSolidColors : useselentrySolidColors));

	if (useselfolderSolidColors)
	{
		selfoldersolidcolors = GetxRCLine(name, "ActiveFolderSolidColors", (activeFolderUseFolderDefault ? foldersolidcolors : selentrysolidcolors));
		if ( !selfoldersolidcolors.empty() )
			parseCFG(selfoldersolidcolors, selfolderbackgroundColor, selfolderlightBevelColor, selfolderdarkBevelColor);
		else
		{
			if ( activeFolderUseFolderDefault )
			{
				selfolderbackgroundColor = GetSysColor(COLOR_BTNFACE);
				selfolderlightBevelColor = GetSysColor(COLOR_3DLIGHT);
				selfolderdarkBevelColor = GetSysColor(COLOR_3DSHADOW);
			}
			else
			{
				selfolderbackgroundColor = GetSysColor(COLOR_BTNHIGHLIGHT);
				selfolderlightBevelColor = GetSysColor(COLOR_3DHILIGHT);
				selfolderdarkBevelColor = GetSysColor(COLOR_3DSHADOW);
			}
		}
		//BG Insert in Gradient List
		SelFolderGradient.insert(SelFolderGradient.end(), selfolderbackgroundColor );

		selfolderbevelSize = GetxRCInt(name, "ActiveFolderSolidBevelSize", (activeFolderUseFolderDefault ? folderbevelSize : selentrybevelSize), 0);
	
		parseGradientCFG(GetxRCLine(name, "ActiveFolderSolidGradientColors", (activeFolderUseFolderDefault ? foldergradientColor : selentrygradientColor)), 4);

		selfolderverticalGradient = GetxRCBool(name, "ActiveFolderSolidGradientVertical", (activeFolderUseFolderDefault ? folderverticalGradient : selentryverticalGradient));
	}
	else
	{
		selfolderImage = GetxRCString(name, "ActiveFolderImage", (activeFolderUseFolderDefault ? folderImage : selentryImage));
		if (!selfolderImage.empty())
		{
			memorySelFolderBitmap = LoadLSImage(selfolderImage.c_str(), 0);

			if (memorySelFolderBitmap && useAlphaMap)
			{
				BITMAP bm;
				GetObject(memorySelFolderBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memorySelFolderBitmap, selfolder.button_saturation, selfolder.button_hueIntensity, selfolder.button_hueColor);
			}

			GetLSBitmapSize(memorySelFolderBitmap, &imageSelFolderWidth, &imageSelFolderHeight);

			memorySelFolderDC = CreateCompatibleDC(NULL);
			SelectObject(memorySelFolderDC, memorySelFolderBitmap);
		}
	}

	useinfoSolidColors = GetxRCBool(name, "UseInfoSolidColors");

	if (useinfoSolidColors)
	{
		infosolidcolors = GetxRCLine(name, "InfoSolidColors", "ffffff");
		parseCFG(infosolidcolors, infobackgroundColor, infolightBevelColor, infodarkBevelColor);
		//BG Insert in Gradient List
		InfoGradient.insert(InfoGradient.end(), infobackgroundColor );

		infobevelSize = GetxRCInt(name, "InfoSolidBevelSize", 0, 0);
	
		parseGradientCFG(GetxRCLine(name, "InfoSolidGradientColors", ""), 5);

		infoverticalGradient = GetxRCBool(name, "InfoSolidGradientVertical");
	}
	else
	{
		infoImage = GetxRCString(name, "InfoImage", "");
		if (!infoImage.empty())
		{
			memoryInfoBitmap = LoadLSImage(infoImage.c_str(), 0);

			if (memoryInfoBitmap && useAlphaMap)
			{
				BITMAP bm;
				GetObject(memoryInfoBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memoryInfoBitmap, info.button_saturation, info.button_hueIntensity, info.button_hueColor);
			}

			GetLSBitmapSize(memoryInfoBitmap, &imageInfoWidth, &imageInfoHeight);

			memoryInfoDC = CreateCompatibleDC(NULL);
			SelectObject(memoryInfoDC, memoryInfoBitmap);
		}
	}

	//Special Textures
	//----------------
	string imageFile = "";

	useseparatorSolidColors = GetxRCBool(name, "UseSeparatorSolidColors");

	if (useseparatorSolidColors)
	{
		imageFile = GetxRCLine(name, "SeparatorSolidColors", "ffffff");
		parseCFG(imageFile, separatorbackgroundColor, separatorlightBevelColor, separatordarkBevelColor);
		//BG Insert in Gradient List
		SeparatorGradient.insert(SeparatorGradient.end(), separatorbackgroundColor );

		separatorbevelSize = GetxRCInt(name, "SeparatorSolidBevelSize", 0, 0);
	
		parseGradientCFG(GetxRCLine(name, "SeparatorSolidGradientColors", ""), 10);

		separatorverticalGradient = GetxRCBool(name, "SeparatorSolidGradientVertical");
	}
	else
	{
		imageFile = GetxRCString(name, "SeparatorImage", "");
		if (!imageFile.empty())
		{
			memorySeparatorBitmap = LoadLSImage(imageFile.c_str(), 0);

			if (memorySeparatorBitmap && useAlphaMap)
			{
				BITMAP bm;
				GetObject(memorySeparatorBitmap, sizeof(BITMAP), &bm);

				if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
					PrepareAlphaChannel(memorySeparatorBitmap, separator.button_saturation, separator.button_hueIntensity, separator.button_hueColor);
			}

			GetLSBitmapSize(memorySeparatorBitmap, &imageSeparatorWidth, &imageSeparatorHeight);

			memorySeparatorDC = CreateCompatibleDC(NULL);
			SelectObject(memorySeparatorDC, memorySeparatorBitmap);
		}
	}

	//Folder Arrows (Normal and Active)
	imageFile = GetxRCString(name, "ArrowImage", "");
	if (!imageFile.empty())
	{
		memoryArrowBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memoryArrowBitmap && useAlphaMap)
		{
			BITMAP bm;
			GetObject(memoryArrowBitmap, sizeof(BITMAP), &bm);

			if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				PrepareAlphaChannel(memoryArrowBitmap, 255, 0, RGB(0,0,0));
		}

		GetLSBitmapSize(memoryArrowBitmap, &imageArrowWidth, &imageArrowHeight);

		arrowSpacing = GetxRCInt(name, "ArrowOffset", 4, 0);
		arrowOnLeft = GetxRCBool(name, "ArrowOnLeft");

		memoryArrowDC = CreateCompatibleDC(NULL);
		SelectObject(memoryArrowDC, memoryArrowBitmap);
	}
	imageFile = GetxRCString(name, "ActiveArrowImage", imageFile);
	if (!imageFile.empty())
	{
		memorySelArrowBitmap = LoadLSImage(imageFile.c_str(), 0);

		if (memorySelArrowBitmap && useAlphaMap)
		{
			BITMAP bm;
			GetObject(memorySelArrowBitmap, sizeof(BITMAP), &bm);

			if ((bm.bmBits != 0) && (bm.bmBitsPixel == 32))
				PrepareAlphaChannel(memorySelArrowBitmap, 255, 0, RGB(0,0,0));
		}

		GetLSBitmapSize(memorySelArrowBitmap, &imageSelArrowWidth, &imageSelArrowHeight);

		selarrowSpacing = GetxRCInt(name, "ActiveArrowOffset", arrowSpacing, 0);
		selarrowOnLeft = GetxRCBool(name, "ActiveArrowOnLeft", arrowOnLeft);

		memorySelArrowDC = CreateCompatibleDC(NULL);
		SelectObject(memorySelArrowDC, memorySelArrowBitmap);
	}	

	if (!useAlphaMap)
	{
		if ((entry.button_saturation != 255 || entry.button_hueIntensity != 0) && !useentrySolidColors)
			paintEffects(memoryEntryDC, imageEntryWidth, imageEntryHeight, entry.button_saturation, entry.button_hueIntensity, entry.button_hueColor);
		if (memorySelEntryDC && (selentry.button_saturation != 255 || selentry.button_hueIntensity != 0) && !useselentrySolidColors)
			paintEffects(memorySelEntryDC, imageSelEntryWidth, imageSelEntryHeight, selentry.button_saturation, selentry.button_hueIntensity, selentry.button_hueColor);
		if (memoryFolderDC && (folder.button_saturation != 255 || folder.button_hueIntensity != 0) && !usefolderSolidColors)
			paintEffects(memoryFolderDC, imageFolderWidth, imageFolderHeight, folder.button_saturation, folder.button_hueIntensity, folder.button_hueColor);
		if (memorySelFolderDC && (selfolder.button_saturation != 255 || selfolder.button_hueIntensity != 0)  && !useselfolderSolidColors)
			paintEffects(memorySelFolderDC, imageSelFolderWidth, imageSelFolderHeight, selfolder.button_saturation, selfolder.button_hueIntensity, selfolder.button_hueColor);
		if (memoryInfoDC && (info.button_saturation != 255 || info.button_hueIntensity != 0)  && !useinfoSolidColors)
			paintEffects(memoryInfoDC, imageInfoWidth, imageInfoHeight, info.button_saturation, info.button_hueIntensity, info.button_hueColor);
		if (memorySeparatorDC && (separator.button_saturation != 255 || separator.button_hueIntensity != 0)  && !useseparatorSolidColors)
			paintEffects(memorySeparatorDC, imageSeparatorWidth, imageSeparatorHeight, separator.button_saturation, separator.button_hueIntensity, separator.button_hueColor);
	}
}

void ButtonTexture::apply(HDC hDC, int x, int y, int width, int height, int buttonType)
{
	RECT r;
	r.left = x;
	r.top = y;
	r.right = r.left + width;
	r.bottom = r.top + height;

	int usedBorderMethod = 0;

	SetStretchBltMode(hDC, STRETCH_DELETESCANS);

	if (buttonType == 1 && useentrySolidColors)
	{
		usedBorderMethod = entry.borderMethod;

		// fill in the background
		if (entrybevelSize > 0 && (entrybevelSize*2 < width && entrybevelSize*2 < height))
		{
			::SetBkColor(hDC, entrydarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-entrybevelSize;
			r.bottom = r.bottom-entrybevelSize;
			::SetBkColor(hDC, entrylightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+entrybevelSize;
			r.top = r.top+entrybevelSize;

			if ( EntryGradient.size() > 1 )
				solidGradientPaint(hDC, EntryGradient, r, entryverticalGradient);
			else
			{
				::SetBkColor(hDC, entrybackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( EntryGradient.size() > 1 )
				solidGradientPaint(hDC, EntryGradient, r, entryverticalGradient);
			else
			{
				::SetBkColor(hDC, entrybackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 2 && useselentrySolidColors )
	{
		usedBorderMethod = selentry.borderMethod;

		// fill in the background
		if (selentrybevelSize > 0 && (selentrybevelSize*2 < width && selentrybevelSize*2 < height))
		{
			::SetBkColor(hDC, selentrydarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-selentrybevelSize;
			r.bottom = r.bottom-selentrybevelSize;
			::SetBkColor(hDC, selentrylightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+selentrybevelSize;
			r.top = r.top+selentrybevelSize;

			if ( SelEntryGradient.size() > 1 )
				solidGradientPaint(hDC, SelEntryGradient, r, selentryverticalGradient);
			else
			{
				::SetBkColor(hDC, selentrybackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SelEntryGradient.size() > 1 )
				solidGradientPaint(hDC, SelEntryGradient, r, selentryverticalGradient);
			else
			{
				::SetBkColor(hDC, selentrybackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 3 && usefolderSolidColors )
	{
		usedBorderMethod = folder.borderMethod;

		// fill in the background
		if (folderbevelSize > 0 && (folderbevelSize*2 < width && folderbevelSize*2 < height))
		{
			::SetBkColor(hDC, folderdarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-folderbevelSize;
			r.bottom = r.bottom-folderbevelSize;
			::SetBkColor(hDC, folderlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+folderbevelSize;
			r.top = r.top+folderbevelSize;

			if ( FolderGradient.size() > 1 )
				solidGradientPaint(hDC, FolderGradient, r, folderverticalGradient);
			else
			{
				::SetBkColor(hDC, folderbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( FolderGradient.size() > 1 )
				solidGradientPaint(hDC, FolderGradient, r, folderverticalGradient);
			else
			{
				::SetBkColor(hDC, folderbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 4 && useselfolderSolidColors )
	{
		usedBorderMethod = selfolder.borderMethod;

		// fill in the background
		if (selfolderbevelSize > 0 && (selfolderbevelSize*2 < width && selfolderbevelSize*2 < height))
		{
			::SetBkColor(hDC, selfolderdarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-selfolderbevelSize;
			r.bottom = r.bottom-selfolderbevelSize;
			::SetBkColor(hDC, selfolderlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+selfolderbevelSize;
			r.top = r.top+selfolderbevelSize;

			if ( SelFolderGradient.size() > 1 )
				solidGradientPaint(hDC, SelFolderGradient, r, selfolderverticalGradient);
			else
			{
				::SetBkColor(hDC, selfolderbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SelFolderGradient.size() > 1 )
				solidGradientPaint(hDC, SelFolderGradient, r, selfolderverticalGradient);
			else
			{
				::SetBkColor(hDC, selfolderbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 5 && useinfoSolidColors )
	{
		usedBorderMethod = info.borderMethod;

		// fill in the background
		if (infobevelSize > 0 && (infobevelSize*2 < width && infobevelSize*2 < height))
		{
			::SetBkColor(hDC, infodarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-infobevelSize;
			r.bottom = r.bottom-infobevelSize;
			::SetBkColor(hDC, infolightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+infobevelSize;
			r.top = r.top+infobevelSize;

			if ( InfoGradient.size() > 1 )
				solidGradientPaint(hDC, InfoGradient, r, infoverticalGradient);
			else
			{
				::SetBkColor(hDC, infobackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( InfoGradient.size() > 1 )
				solidGradientPaint(hDC, InfoGradient, r, infoverticalGradient);
			else
			{
				::SetBkColor(hDC, infobackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else if ( buttonType == 10 && useseparatorSolidColors )
	{
		usedBorderMethod = separator.borderMethod;

		// fill in the background
		if (separatorbevelSize > 0 && (separatorbevelSize*2 < width && separatorbevelSize*2 < height))
		{
			::SetBkColor(hDC, separatordarkBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.right = r.right-separatorbevelSize;
			r.bottom = r.bottom-separatorbevelSize;
			::SetBkColor(hDC, separatorlightBevelColor);
			::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);

			r.left = r.left+separatorbevelSize;
			r.top = r.top+separatorbevelSize;

			if ( SeparatorGradient.size() > 1 )
				solidGradientPaint(hDC, SeparatorGradient, r, separatorverticalGradient);
			else
			{
				::SetBkColor(hDC, separatorbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
		else
		{
			if ( SeparatorGradient.size() > 1 )
				solidGradientPaint(hDC, SeparatorGradient, r, separatorverticalGradient);
			else
			{
				::SetBkColor(hDC, separatorbackgroundColor);
				::ExtTextOut(hDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
			}
		}
	}
	else
	{
		if ( buttonType == 31 && memoryArrowBitmap == 0 )
			return;
		if ( buttonType == 41 && memorySelArrowBitmap == 0 )
			return;

		if ( buttonType == 10 && memorySeparatorBitmap == 0 )
			return;

		if ( buttonType == 5 && memoryInfoBitmap == 0 )
			return;
		if ( buttonType == 4 && memorySelFolderBitmap == 0 )
			buttonType = (activeFolderUseFolderDefault ? 3 : 2);		
		if ( buttonType == 3 && memoryFolderBitmap == 0 )
			buttonType = 1;	
		if ( buttonType == 2 && memorySelEntryBitmap == 0 )
			buttonType = 1;
		
		if (buttonType == 1 && memoryEntryBitmap == 0)
			return;

		bool useTrueTransparent = false;

		HDC bufferDC = CreateCompatibleDC(hDC);
		HBITMAP bufferBitmap = CreateCompatibleBitmap(hDC, width, height);
		SelectObject(bufferDC, bufferBitmap);

		SetStretchBltMode(bufferDC, STRETCH_DELETESCANS);

		if ( buttonType == 31 || buttonType == 41 )
		{
			r.left = 0;
			r.top = 0;
			r.right = width;
			r.bottom = height;

			::SetBkColor(bufferDC, RGB(255,0,255));
			::ExtTextOut(bufferDC, 0, 0, ETO_OPAQUE, &r, NULL, 0, NULL);
		}
	
		if (buttonType == 1)
		{
			usedBorderMethod = entry.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryEntryDC, 0, 0, imageEntryWidth, imageEntryHeight,
					entry.leftEdge, entry.topEdge, entry.rightEdge, entry.bottomEdge,
					entry.mode, SRCCOPY);

			useTrueTransparent = entry.trueTransparent;
		}
		else if (buttonType == 2)
		{
			usedBorderMethod = selentry.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memorySelEntryDC, 0, 0, imageSelEntryWidth, imageSelEntryHeight,
					selentry.leftEdge, selentry.topEdge, selentry.rightEdge, selentry.bottomEdge,
					selentry.mode, SRCCOPY);

			useTrueTransparent = selentry.trueTransparent;

		}
		else if (buttonType == 3)
		{
			usedBorderMethod = folder.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryFolderDC, 0, 0, imageFolderWidth, imageFolderHeight,
					folder.leftEdge, folder.topEdge, folder.rightEdge, folder.bottomEdge,
					folder.mode, SRCCOPY);

			useTrueTransparent = folder.trueTransparent;
		}
		else if (buttonType == 4)
		{
			usedBorderMethod = selfolder.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memorySelFolderDC, 0, 0, imageSelFolderWidth, imageSelFolderHeight,
					selfolder.leftEdge, selfolder.topEdge, selfolder.rightEdge, selfolder.bottomEdge,
					selfolder.mode, SRCCOPY);

			useTrueTransparent = selfolder.trueTransparent;
		}
		else if (buttonType == 5)
		{
			usedBorderMethod = info.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
					memoryInfoDC, 0, 0, imageInfoWidth, imageInfoHeight,
					info.leftEdge, info.topEdge, info.rightEdge, info.bottomEdge,
					info.mode, SRCCOPY);

			useTrueTransparent = info.trueTransparent;
		}
		else if (buttonType == 10)
		{
			usedBorderMethod = separator.borderMethod;

			MultiBlt(bufferDC, 0, 0, width, height,
						memorySeparatorDC, 0, 0, imageSeparatorWidth, imageSeparatorHeight,
						separator.leftEdge, separator.topEdge, separator.rightEdge, separator.bottomEdge,
						separator.mode, SRCCOPY);

			useTrueTransparent = separator.trueTransparent;
		}
		else if ( buttonType == 31 )
		{
			if (useAlphaMap)
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

				AlphaBlend(hDC, (arrowOnLeft ? x+arrowSpacing : x+width-imageArrowWidth-arrowSpacing), y+(height-imageArrowHeight)/2, imageArrowWidth, imageArrowHeight, memoryArrowDC, 0, 0, imageArrowWidth, imageArrowHeight, blendFunction);
			
				bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
				DeleteObject(bufferBitmap);
				DeleteDC(bufferDC);

				return;
			}
			else
			{
				MultiBlt(bufferDC, (arrowOnLeft ? arrowSpacing : width-imageArrowWidth-arrowSpacing), (height-imageArrowHeight)/2, imageArrowWidth, imageArrowHeight,
						memoryArrowDC, 0, 0, imageArrowWidth, imageArrowHeight,
						0, 0, 0, 0,
						MULTIBLT_STRETCH, SRCCOPY);
			}
		}
		else if ( buttonType == 41 )
		{
			if (useAlphaMap)
			{
				BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

				AlphaBlend(hDC, (selarrowOnLeft ? x+selarrowSpacing : x+width-imageSelArrowWidth-selarrowSpacing), y+(height-imageSelArrowHeight)/2, imageSelArrowWidth, imageSelArrowHeight, memorySelArrowDC, 0, 0, imageSelArrowWidth, imageSelArrowHeight, blendFunction);
				
				bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
				DeleteObject(bufferBitmap);
				DeleteDC(bufferDC);

				return;
			}
			else
			{
				MultiBlt(bufferDC, (selarrowOnLeft ? selarrowSpacing : width-imageSelArrowWidth-selarrowSpacing), (height-imageSelArrowHeight)/2, imageSelArrowWidth, imageSelArrowHeight,
						memorySelArrowDC, 0, 0, imageSelArrowWidth, imageSelArrowHeight,
						0, 0, 0, 0,
						MULTIBLT_STRETCH, SRCCOPY);
			}
		}
	
		if (useAlphaMap)
		{
			BLENDFUNCTION blendFunction = { AC_SRC_OVER, 0, 255, (useTrueTransparent ? 0 : AC_SRC_ALPHA) };

			AlphaBlend(hDC, x, y, width, height, bufferDC, 0, 0, width, height, blendFunction);
		}
		else
		{
			SetStretchBltMode(hDC, STRETCH_DELETESCANS);

			if ( useTrueTransparent )
			{
				BitBlt(hDC, x, y, width, height, bufferDC, 0, 0, SRCCOPY);
			}
			else
			{
				TransparentBltLS(hDC, x, y, width, height,
									bufferDC, 0, 0,
									RGB(255, 0, 255));
			}
		}

		bufferBitmap = (HBITMAP) SelectObject(bufferDC, bufferBitmap);
		DeleteObject(bufferBitmap);
		DeleteDC(bufferDC);
	}

	if ( !useAlphaMap )
	{
		RECT completeFillRect = {x, y, x+width, y+height};
		if (usedBorderMethod == 1)
			DrawEdge(hDC, &completeFillRect, EDGE_RAISED, BF_RECT | BF_SOFT);
		else if (usedBorderMethod == 2)
			DrawEdge(hDC, &completeFillRect, EDGE_SUNKEN, BF_RECT | BF_SOFT);
		else if (usedBorderMethod == 3)
			DrawEdge(hDC, &completeFillRect, EDGE_ETCHED, BF_RECT | BF_SOFT);
		else if (usedBorderMethod == 4)
			DrawEdge(hDC, &completeFillRect, EDGE_BUMP, BF_RECT | BF_SOFT);
	}
}

void ButtonTexture::paintEffects(HDC destHDC, int width, int height, int button_saturation, int button_hueIntensity, COLORREF button_hueColor)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	int iy = 0;
	int ix = 0;
	COLORREF cl = 0;
	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for(iy = 0; iy < height; iy++)
	{
		for(ix = 0; ix < width; ix++)
		{
			cl = GetPixel(destHDC, ix, iy);
			r = GetRValue(cl);
			g = GetGValue(cl);
			b = GetBValue(cl);
		
			if( !( r == 255 && g == 0 && b == 255 ) )
			{
				// Saturation effect
				if(button_saturation != 255)
				{
					BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);

					r = (BYTE)((r * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
					g = (BYTE)((g * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
					b = (BYTE)((b * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
				}

				// Hue effect
				if(button_hueIntensity != 0)
				{
					// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
					// test normal
					r = (BYTE)((r * (255 - button_hueIntensity) + GetRValue(button_hueColor) * button_hueIntensity + 255) >> 8);
					g = (BYTE)((g * (255 - button_hueIntensity) + GetGValue(button_hueColor) * button_hueIntensity + 255) >> 8);
					b = (BYTE)((b * (255 - button_hueIntensity) + GetBValue(button_hueColor) * button_hueIntensity + 255) >> 8);
				}
			}

			SetPixelV(destHDC, ix, iy, RGB(r, g, b));
		}
	}
	//End Saturation / Hueing
}

void ButtonTexture::solidGradientPaint(HDC destHDC, ColorRefList workGradient, RECT r, bool vertical)
{
	int nWidth = r.right - r.left;
	int nHeight = r.bottom - r.top;

	RECT r2;
	double steps = 1;
	double cr, cg, cb, dr, dg, db;
	int gradSteps;

	int colorCount = workGradient.size();

	if (vertical)
		gradSteps = max(nHeight / (colorCount-1), 1);
	else
		gradSteps = max(nWidth / (colorCount-1), 1);

	int lastGradientPos;
	if (vertical)
		lastGradientPos = r.top;
	else
		lastGradientPos = r.left;

	for (ColorRefListIterator cit = workGradient.begin(); cit != workGradient.end(); cit++ )
	{
		if (cit == workGradient.end())
			break;

		COLORREF one = *cit;
		cit++;
		COLORREF two = *cit;
		cit--;
	
		// get current color
		cr = GetRValue( one );
		cg = GetGValue( one );
		cb = GetBValue( one );

		// Get delta color
		dr = (cr - GetRValue( two )) / gradSteps;
		dg = (cg - GetGValue( two )) / gradSteps;
		db = (cb - GetBValue( two )) / gradSteps;

		for (int i = 1; i <= gradSteps; i++)
		{
			cr -= dr;
			cg -= dg;
			cb -= db;
		
			if (vertical)
				SetRect(&r2, r.left, (int)(lastGradientPos+steps*(i - 1)), r.right, min((int)(lastGradientPos+steps*i), r.bottom));
			else
				SetRect(&r2, (int)(lastGradientPos+steps*(i - 1)), r.top, min((int)(lastGradientPos+steps*i), r.right), r.bottom);

			if (r2.right-r2.left > 0 && r2.bottom-r2.top > 0)
			{
				::SetBkColor(destHDC, RGB((int)cr, (int)cg, (int)cb));
				::ExtTextOut(destHDC, 0, 0, ETO_OPAQUE, &r2, NULL, 0, NULL);
			}
		}
		if (vertical)
			lastGradientPos = r2.bottom;
		else
			lastGradientPos = r2.right;
	}
}

void ButtonTexture::PrepareAlphaChannel(HBITMAP &bmp, int button_saturation, int button_hueIntensity, COLORREF button_hueColor)
{
	//Paint Saturation and Hueing Effects in MemoryDC's
	BITMAP bm;
	GetObject(bmp, sizeof(BITMAP), &bm);

	BYTE *pPixel = (LPBYTE)bm.bmBits;

	BYTE r = 0;
	BYTE g = 0;
	BYTE b = 0;

	for (int i = 0; i < bm.bmHeight * bm.bmWidth; ++i)
	{
		//Magic Pink
		if (pPixel[0] == 255 && pPixel[1] == 0 && pPixel[2] == 255)
			pPixel[3] = 0;

		//Prepare Text
		if (pPixel[3] == 0)
			pPixel[3] = 1;

		r = pPixel[0];
		g = pPixel[1];
		b = pPixel[2];

		//AlphaMap Colorizing
		//Saturation Effect
		if( button_saturation != 255 )
		{
			BYTE gray = (BYTE)(r * 0.3086 + g * 0.6094 + b * 0.0820);
			r = (BYTE)((r * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
			g = (BYTE)((g * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
			b = (BYTE)((b * button_saturation + gray * (255 - button_saturation) + 255) >> 8);
		}
		// Hue effect
		if( button_hueIntensity != 0 )
		{
			// The B & R values of the hue is swapped here, can somebody tell me why it only works that way?
			// test normal
			b = (BYTE)((r * (255 - button_hueIntensity) + GetRValue(button_hueColor) * button_hueIntensity + 255) >> 8);
			g = (BYTE)((g * (255 - button_hueIntensity) + GetGValue(button_hueColor) * button_hueIntensity + 255) >> 8);
			r = (BYTE)((b * (255 - button_hueIntensity) + GetBValue(button_hueColor) * button_hueIntensity + 255) >> 8);
		}
		
		//PreMultiply
		pPixel[0] = r * pPixel[3] / 255;
		pPixel[1] = g * pPixel[3] / 255;
		pPixel[2] = b * pPixel[3] / 255;

		pPixel+= 4;
	}
}